"""
Data models
"""
from box import Box, BoxList


class BaseModel(Box):
    """
    Base model.
    """

    def __init__(self, *args, **kwargs):
        kwargs.update({
            "camel_killer_box": True,
        })
        super().__init__(*args, **kwargs)

class BaseListModel(BoxList):
    """
    Base model.
    """

    def __init__(self, *args, **kwargs):
        kwargs.update({
            "camel_killer_box": True,
        })
        super().__init__(*args, **kwargs)


class Clan(BaseModel):
    """Clan."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class TopClans(BaseModel):
    """Clan."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Player(BaseModel):
    """Player profile."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def clan_name(self):
        return self.clan.name

    @property
    def clan_role(self):
        return self.clan.role

class TopPlayers(BaseListModel):
    """Top Players"""
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Constants(BaseModel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_chest_by_index(self, index):
        """Return chest by index."""
        order = self.chest_cycle.order
        return order[index % len(order)]

    def get_card(self, key=None, card_key=None, name=None):
        """Return card by any property"""
        for card in self.cards:
            if card.key == key:
                return card
            if card.card_key == card_key:
                return card
            if card.name == name:
                return card
        return None


class Tag:
    """SuperCell tags."""

    TAG_CHARACTERS = "0289PYLQGRJCUV"

    def __init__(self, tag: str):
        """Init.

        Remove # if found.
        Convert to uppercase.
        Convert Os to 0s if found.
        """
        if tag.startswith('#'):
            tag = tag[1:]
        tag = tag.replace('O', '0')
        tag = tag.upper()
        self._tag = tag

    def __str__(self):
        return self._tag

    def __repr__(self):
        return self._tag

    @property
    def tag(self):
        """Return tag as str."""
        return self._tag

    @property
    def valid(self):
        """Return true if tag is valid."""
        for c in self.tag:
            if c not in self.TAG_CHARACTERS:
                return False
        return True

    @property
    def invalid_chars(self):
        """Return list of invalid characters."""
        invalids = []
        for c in self.tag:
            if c not in self.TAG_CHARACTERS:
                invalids.append(c)
        return invalids



import pytest

from crapipy import AsyncClient, Client


@pytest.mark.asyncio
async def test_clans_async():
    client = AsyncClient()
    clans = await client.get_clans(['2CCCP', '2U2GGQJ'])
    assert clans[0].name == 'Reddit Alpha'
    assert clans[0].badge.name == 'A_Char_Rocket_02'
    assert clans[1].name == 'Reddit Bravo'
    assert clans[1].badge.name == 'A_Char_Rocket_02'

def test_clans():
    client = Client()
    clans = client.get_clans(['2CCCP', '2U2GGQJ'])
    assert clans[0].name == 'Reddit Alpha'
    assert clans[0].badge.name == 'A_Char_Rocket_02'
    assert clans[1].name == 'Reddit Bravo'
    assert clans[1].badge.name == 'A_Char_Rocket_02'





class APIURL:
    """
    API URL
    """
    clan = 'http://api.cr-api.com/clan/{}'
    top_clans = 'http://api.cr-api.com/top/clans'
    player = 'http://api.cr-api.com/player/{}'
    constants = 'http://api.cr-api.com/constants'
    top_players = 'http://api.cr-api.com/top/players'



import pytest

from crapipy import AsyncClient


@pytest.mark.asyncio
async def test_profiles():
    client = AsyncClient()
    players = await client.get_players(['C0G20PR2', 'PY9VC98C'])
    assert players[0].name == 'SML'
    assert players[0].tag == 'C0G20PR2'
    assert players[0].clan.name == 'Reddit Delta'
    assert players[0].clan.role.lower() == 'leader'
    assert players[1].name == 'Selfish'
    assert players[1].tag == 'PY9VC98C'
    assert players[1].clan.name == 'Reddit Gents.'
    assert players[1].clan.role.lower() == 'leader'



from crapipy import Client

client = Client()
top_players = client.get_top_players()
print(top_players)
print(top_players.last_updated)
print(top_players.players)
print(top_players.players[0].clan.name)
print(top_players.players[0].trophies)


import pytest

from crapipy import AsyncClient, Client, APIError


@pytest.mark.asyncio
async def test_clan_async():
    client = AsyncClient()
    clan = await client.get_clan('2CCCP')
    assert clan.name == 'Reddit Alpha'
    assert clan.badge.name == 'A_Char_Rocket_02'

def test_clan():
    client = Client()
    clan = client.get_clan('2CCCP')
    assert clan.name == 'Reddit Alpha'
    assert clan.badge.name == 'A_Char_Rocket_02'

@pytest.mark.asyncio
async def test_invalid_clan_async():
    client = AsyncClient()
    with pytest.raises(APIError):
        clan = await client.get_clan('123445')

def test_invalid_clan():
    client = Client()
    with pytest.raises(APIError):
        clan = client.get_clan('123445')





"""
cr-api client for Clash Royale.
"""
import json
import logging
import os

import requests
from requests.exceptions import HTTPError

from .exceptions import APIError
from .models import Clan, TopClans, Player, Constants, Tag, TopPlayers
from .url import APIURL

logger = logging.getLogger('__name__')
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.ERROR)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)


class Client:
    """
    API Client.
    """

    def __init__(self, token=None):
        self._token = token

    @property
    def token(self):
        """Load token from environment if not defined"""
        if self._token is None:
            self._token = os.environ['TOKEN']
        return self._token

    def fetch(self, url):
        """Fetch URL.

        :param url: URL
        :return: Response in JSON

        """
        headers = {'auth': self.token}
        try:
            r = requests.get(url, headers=headers)
            data = r.json()

            if r.status_code != 200:
                logger.error(
                    "API Error | HTTP status {status} | url: {url}".format(
                        status=r.status_code,
                        url=url
                    )
                )
                raise APIError(**data)

            if isinstance(data, dict):
                if data.get('error'):
                    raise APIError(**data)

        except (HTTPError, ConnectionError, json.JSONDecodeError):
            raise APIError

        return data

    def get_clan(self, clan_tag):
        """Fetch a single clan."""
        url = APIURL.clan.format(clan_tag)
        data = self.fetch(url)
        return Clan(data)

    def get_clans(self, clan_tags):
        """Fetch multiple clans.

        :param clan_tags: List of clan tags
        """
        url = APIURL.clan.format(','.join(clan_tags))
        data = self.fetch(url)
        return [Clan(d) for d in data]

    def get_top_clans(self):
        """Fetch top clans."""
        data = self.fetch(APIURL.top_clans)
        return TopClans(data)

    def get_profile(self, tag: str):
        """Get player profile by tag.
        :param tag:
        :return:
        """
        ptag = Tag(tag).tag
        url = APIURL.player.format(ptag)
        data = self.fetch(url)
        return Player(data)

    def get_profiles(self, tags):
        """Fetch multiple players from profile API."""
        ptags = [Tag(tag).tag for tag in tags]
        url = APIURL.player.format(','.join(ptags))
        data = self.fetch(url)
        return [Player(d) for d in data]

    def get_constants(self, key=None):
        """Fetch contants.

        :param key: Optional field.
        """
        url = APIURL.constants
        data = self.fetch(url)
        return Constants(data)

    def get_top_players(self):
        """Fetch top players."""
        url = APIURL.top_players
        data = self.fetch(url)
        return TopPlayers(data)



"""
Utility functions
"""

from box import Box, BoxList

def make_box(data):
    """Create Box instance from dict."""
    return Box(data, camel_killer_box=True)

def make_box_list(data):
    """Create BoxList instance from dict."""
    return BoxList(data, camel_killer_box=True)



import pytest

from crapipy import AsyncClient
import pprint


@pytest.mark.asyncio
async def test_constants():
    pass
    # TODO: Removed as there are no constants til later.
    # client = AsyncClient()
    # constants = await client.get_constants()
    # pp = pprint.PrettyPrinter(indent=4)
    # assert constants.arenas[0].arena == 'Arena 1'
    # assert constants.badges["16000000"] == 'Flame_01'
    # assert constants.chest_cycle.order[0] == 'Silver'
    # assert constants.get_chest_by_index(3) == 'Gold'
    #
    # archers = constants.get_card(key='archers')
    # assert archers.elixir == 3
    # assert archers.name == 'Archers'
    # assert archers.rarity == 'Common'

    # assert constants.get_region_named("Europe").is_country == False
    # assert constants.get_region_named("Swaziland").is_country == True





import pytest

from crapipy import AsyncClient, Client


@pytest.mark.asyncio
async def test_top_players_async():
    client = AsyncClient()
    top_players = await client.get_top_players()
    assert len(top_players) > 0
    assert top_players[0].rank == 1


def test_top_players():
    client = Client()
    top_players = client.get_top_players()
    assert len(top_players) > 0
    assert top_players[0].rank == 1



"""
Clash Royale wrapper for cr-api.com
"""
__version__ = "1.1"

from .client import Client
from .client_async import AsyncClient
from .exceptions import APITimeoutError, APIClientResponseError, APIError
from .url import APIURL
from .models import Clan, TopClans, TopPlayers, Tag


"""
Wrapper exceptions
"""

class BaseException(Exception):
    def __init__(self):
        super().__init__()

class APIError(BaseException):
    def __init__(self, error=None, status=None, message=None):
        super().__init__()
        self.error = error
        self.status = status
        self.message = message



class APITimeoutError(APIError):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class APIClientResponseError(APIError):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)




import pytest

from crapipy import AsyncClient, Client


def assert_player_model(player):
    assert player.name == 'SML'
    assert player.tag == 'C0G20PR2'
    assert player.clan.name == 'Reddit Delta'
    assert player.clan.role.lower() == 'leader'
    assert player.clan_name == player.clan.name
    assert player.clan_role == player.clan.role
    assert player.current_deck is not None
    assert player.arena.arena_id == player.arena.arenaID
    assert player.arena.arena_id is not None


@pytest.mark.asyncio
async def test_profile_async():
    client = AsyncClient()
    player = await client.get_player('C0G20PR2')
    assert_player_model(player)


def test_profile():
    client = Client()
    player = client.get_profile('C0G20PR2')
    assert_player_model(player)


@pytest.mark.asyncio
async def test_profile_equal():
    client = AsyncClient()
    player1 = await client.get_player('C0G20PR2')
    player2 = await client.get_player('C0G20PR2')
    assert player1 == player2


@pytest.mark.asyncio
async def test_profile_not_equal():
    client = AsyncClient()
    player1 = await client.get_player('C0G20PR2')
    player2 = await client.get_player('PY9VC98C')
    assert player1 != player2



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['crapipy']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=2.2.5',
 'async-timeout>=2.0.0',
 'asyncio>=3.4.3',
 'pytest>=3.2.3',
 'pytest-asyncio>=0.8.0',
 'python-box>=3.1.1',
 'PyYAML>=3.12',
 'requests>=2.18.4']

setup(name='crapipy',
      version='1.1',
      description='Clash Royale wrapper for cr-api.com',
      author='SML BioBot',
      author_email='smlbiobot@gmail.com',
      url='http://github.com/smlbiobot/cr-api-py',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.5',
     )



"""
cr-api async client for Clash Royale.
"""
import asyncio
import json
import logging
import os

import aiohttp

from .exceptions import APIError
from .models import Clan, Tag, Player, Constants, TopPlayers
from .url import APIURL
from .util import make_box

logger = logging.getLogger('__name__')
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.ERROR)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)


class AsyncClient:
    """
    API AsyncClient.
    """

    def __init__(self, token=None):
        self._token = token

    @property
    def token(self):
        """Load token from environment if not defined"""
        if self._token is None:
            self._token = os.environ['TOKEN']
        return self._token

    async def fetch(self, url):
        """Fetch URL.

        :param url: URL
        :return: Response in JSON
        """
        headers = {'auth': self.token}
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url, headers=headers) as resp:
                    data = await resp.json()
                    if resp.status != 200:
                        logger.error(
                            "API Error | HTTP status {status} | {errmsg} | url: {url}".format(
                                status=resp.status,
                                errmsg=data.get('error'),
                                url=url
                            )
                        )
                        raise APIError(**data)

        except (asyncio.TimeoutError, aiohttp.ClientResponseError, json.JSONDecodeError):
            raise APIError

        return data

    async def get_clan(self, clan_tag):
        """Fetch a single clan."""
        url = APIURL.clan.format(clan_tag)
        data = await self.fetch(url)
        if isinstance(data, list):
            data = data[0]
        return Clan(data)

    async def get_clans(self, clan_tags):
        """Fetch multiple clans.

        :param clan_tags: List of clan tags
        """
        url = APIURL.clan.format(','.join(clan_tags))
        data = await self.fetch(url)
        return [Clan(d) for d in data]

    async def get_top_clans(self):
        """Fetch top clans."""
        data = await self.fetch(APIURL.top_clans)
        return make_box(data)

    async def get_player(self, tag: str) -> Player:
        """Get player profile by tag.
        :param tag: 
        :return: 
        """
        ptag = Tag(tag).tag
        url = APIURL.player.format(ptag)
        data = await self.fetch(url)
        return Player(data)

    async def get_players(self, tags):
        """Fetch multiple players from profile API."""
        ptags = [Tag(tag).tag for tag in tags]
        url = APIURL.player.format(','.join(ptags))
        data = await self.fetch(url)
        return [Player(d) for d in data]

    async def get_constants(self, key=None):
        """Fetch contants.

        :param key: Optional field.
        """
        url = APIURL.constants
        data = await self.fetch(url)
        return Constants(data)

    async def get_top_players(self):
        """Fetch top players."""
        url = APIURL.top_players
        data = await self.fetch(url)
        return TopPlayers(data)
